#include "adi_dmc_config_generator.h"

#include <sys/platform.h>
#include <stdio.h>
#include <math.h>

static ADI_DMC_RESULT SetDdrCl(uint32_t cl,ADI_DMC_PARAM_LIST_INT* Dmc_param);
static ADI_DMC_RESULT SetDdrWrrec(uint32_t wrrec,ADI_DMC_PARAM_LIST_INT* Dmc_param);
static ADI_DMC_RESULT SetMemDRV(uint32_t MemDrv,ADI_DMC_PARAM_LIST_INT* Dmc_param);
static ADI_DMC_RESULT SetMemODT(uint32_t MemOdt,ADI_DMC_PARAM_LIST_INT* Dmc_param);

static float getTwtr(float tck);
static float getTmrd();
static float getTrrd(float tck,int iterator);
static float getTrtp(float tck);
static float getTxp(float tck,int iterator);
static float getTcke(float tck,int iterator);
static float trfcTable[NUM_MEM_SIZE]={90,110,160,260,350};

float trefiTable[NUM_TEMP][NUM_MEM_SIZE]=
{
		{7800,7800,7800,7800,7800},
		{3900,3900,3900,3900,3900}
};

float speedbinTimingTable[NUM_SPEEDBIN_TIMING][NUM_SPEED_BIN]=
{
		{15,13.125,13.5,13.75,13.91,13.09},
		{52.5,50.625,49.5,48.75,47.91,46.09},
		{37.5,37.5,36,35,34,33},
		{6,7,9,11,13,14},
		{5,6,7,8,9,10},
		{50,50,45,40,35,35},
		{15,15,15,15,15,15},
		{4,4,4,4,4,4}
};

static ADI_DMC_RESULT SetMemODT(uint32_t MemOdt,ADI_DMC_PARAM_LIST_INT* Dmc_param)
{
	ADI_DMC_RESULT eResult= ADI_DMC_SUCCESS;

	if(MemOdt == ADI_DDR3_ODT_60)
	{
		Dmc_param->rtt0 = 1u;
		Dmc_param->rtt1 = 0u;
		Dmc_param->rtt2 = 0u;
	}
	else if(MemOdt == ADI_DDR3_ODT_120)
	{
		Dmc_param->rtt0 = 0u;
		Dmc_param->rtt1 = 1u;
		Dmc_param->rtt2 = 0u;
	}
	else if(MemOdt == ADI_DDR3_ODT_40)
	{
		Dmc_param->rtt0 = 1u;
		Dmc_param->rtt1 = 1u;
		Dmc_param->rtt2 = 0u;
	}
	else if(MemOdt == ADI_DDR3_ODT_20)
	{
		Dmc_param->rtt0 = 0u;
		Dmc_param->rtt1 = 0u;
		Dmc_param->rtt2 = 1u;
	}
	else if(MemOdt == ADI_DDR3_ODT_30)
	{
		Dmc_param->rtt0 = 1u;
		Dmc_param->rtt1 = 0u;
		Dmc_param->rtt2 = 1u;
	}
	else
	{
		eResult= ADI_DMC_FAILURE;
	}

	return eResult;
}


static ADI_DMC_RESULT SetMemDRV(uint32_t MemDrv,ADI_DMC_PARAM_LIST_INT* Dmc_param)
{
	ADI_DMC_RESULT eResult= ADI_DMC_SUCCESS;


	if(MemDrv == ADI_DDR3_DRV_40)
	{
		Dmc_param->dic0 = 0u;
		Dmc_param->dic1 = 0u;
	}
	else if(MemDrv == ADI_DDR3_DRV_34)
	{
		Dmc_param->dic0 = 1u;
		Dmc_param->dic1 = 0u;
	}
	else
	{
		eResult= ADI_DMC_FAILURE;
	}

	return eResult;
}


static ADI_DMC_RESULT SetDdrCl(uint32_t cl,ADI_DMC_PARAM_LIST_INT* Dmc_param)
{
	ADI_DMC_RESULT eResult= ADI_DMC_SUCCESS;

	switch (cl)
	{
	case 5u: Dmc_param->cl0=0u;
	Dmc_param->cl=1u;
	break;

	case 6u: Dmc_param->cl0=0u;
	Dmc_param->cl=2u;
	break;

	case 7u: Dmc_param->cl0=0u;
	Dmc_param->cl=3u;
	break;

	case 8u: Dmc_param->cl0=0u;
	Dmc_param->cl=4u;
	break;

	case 9u: Dmc_param->cl0=0u;
	Dmc_param->cl=5u;
	break;

	case 10u: Dmc_param->cl0=0u;
	Dmc_param->cl=6u;
	break;

	case 11u: Dmc_param->cl0=0u;
	Dmc_param->cl=7u;
	break;

	case 12u: Dmc_param->cl0=1u;
	Dmc_param->cl=0u;
	break;

	case 13u: Dmc_param->cl0=1u;
	Dmc_param->cl=1u;
	break;

	case 14u: Dmc_param->cl0=1u;
	Dmc_param->cl=2u;
	break;

	default: eResult= ADI_DMC_FAILURE;
	break;

	}
	return eResult;
}


static ADI_DMC_RESULT SetDdrWrrec(uint32_t wrrec,ADI_DMC_PARAM_LIST_INT* Dmc_param)
{
	ADI_DMC_RESULT eResult= ADI_DMC_SUCCESS;

	switch (wrrec)
	{
	case 5u: Dmc_param->wrrec=1u;
	break;

	case 6u: Dmc_param->wrrec=2u;
	break;

	case 7u: Dmc_param->wrrec=3u;
	break;

	case 8u: Dmc_param->wrrec=4u;
	break;
	case 9u:
	case 10u: Dmc_param->wrrec=5u;
	break;

	case 11u:
	case 12u: Dmc_param->wrrec=6u;
	break;

	case 13u:
	case 14u: Dmc_param->wrrec=7u;
	break;
	default: eResult= ADI_DMC_FAILURE;
	break;
	}

	return eResult;
}



static float getMax(float a, float b)
{
	if(a>b)
	{
		return a;
	}
	else
	{
		return b;
	}
}


static float getTwtr(float tck)
{
	return (getMax(4,7.5/tck));
}


static float getTrrd(float tck,int iterator)
{
    switch (iterator)
    {
    	case 0:
    	case 1: return (getMax(4,10/tck));
    	case 2:
    	case 3:return getMax(4,7.5/tck);
    	case 4:
    	case 5:return getMax(4,6.0/tck);
    	default:return -1;

    }

}

static float getTrtp(float tck)
{
	return getMax(4,7.5/tck);
}

static float getTxp(float tck,int iterator)
{
    switch (iterator)
    {
    	case 0:
    	case 1: return getMax(3,7.5/tck);
    	case 2:
    	case 3:
    	case 4:
    	case 5:return getMax(3,6/tck);
    	default: return -1;
    }
}

static float getTcke(float tck,int iterator)
{
    switch (iterator)
    {
    	case 0:	return getMax(3,7.5/tck);
    	case 1:
    	case 2: return getMax(3,5.625/tck);
    	case 3:
    	case 4:
    	case 5:return getMax(3,5/tck);
    	default: return -1;
    }
}


ADI_DMC_RESULT adi_DMCinit(ADI_DMC_PARAM_LIST* Dmc_config)
{

	ADI_DMC_PARAM_LIST_INT Dmc_param;
	ADI_DMC_CONFIG config;
	ADI_DMC_RESULT eResult= ADI_DMC_SUCCESS;

	Dmc_param.trcd = speedbinTimingTable[ADI_DDR3_MEM_TRCD_TRP][Dmc_config->DDRSpeedBin];
	Dmc_param.trp = speedbinTimingTable[ADI_DDR3_MEM_TRCD_TRP][Dmc_config->DDRSpeedBin];
	Dmc_param.tras = speedbinTimingTable[ADI_DDR3_MEM_TRAS][Dmc_config->DDRSpeedBin];
	Dmc_param.trc = speedbinTimingTable[ADI_DDR3_MEM_TRC][Dmc_config->DDRSpeedBin];
	Dmc_param.tmrd = speedbinTimingTable[ADI_DDR3_MEM_TMRD][Dmc_config->DDRSpeedBin];
	Dmc_param.trefi = trefiTable[Dmc_config->MemTemp][Dmc_config->MemSize];
	Dmc_param.trfc = trfcTable[Dmc_config->MemSize];
	Dmc_param.tfaw = speedbinTimingTable[ADI_DDR3_MEM_TFAW][Dmc_config->DDRSpeedBin];
	Dmc_param.twr = speedbinTimingTable[ADI_DDR3_MEM_TWR][Dmc_config->DDRSpeedBin];
	Dmc_param.cl = speedbinTimingTable[ADI_DDR3_MEM_CL][Dmc_config->DDRSpeedBin];
	Dmc_param.cwl = speedbinTimingTable[ADI_DDR3_MEM_CWL][Dmc_config->DDRSpeedBin];
	Dmc_param.datacyc = 15/Dmc_config->DDRClockTimePeriod;
	Dmc_param.al = ADI_DDR3_AL_0;

	Dmc_param.twtr = getTwtr(Dmc_config->DDRClockTimePeriod);
	Dmc_param.trrd = getTrrd(Dmc_config->DDRClockTimePeriod,Dmc_config->DDRSpeedBin);
	Dmc_param.trtp = getTrtp(Dmc_config->DDRClockTimePeriod);
	Dmc_param.tcke = getTcke(Dmc_config->DDRClockTimePeriod,Dmc_config->DDRSpeedBin);
	Dmc_param.txp = getTxp(Dmc_config->DDRClockTimePeriod,Dmc_config->DDRSpeedBin);


	Dmc_param.trcd = (uint32_t)ceil(Dmc_param.trcd/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trp = (uint32_t)ceil(Dmc_param.trp/Dmc_config->DDRClockTimePeriod);
	Dmc_param.tras = (uint32_t)ceil(Dmc_param.tras/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trc = (uint32_t)ceil(Dmc_param.trc/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trefi = (uint32_t)ceil(Dmc_param.trefi/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trfc = (uint32_t)ceil(Dmc_param.trfc/Dmc_config->DDRClockTimePeriod);
	Dmc_param.tfaw = (uint32_t)ceil(Dmc_param.tfaw/Dmc_config->DDRClockTimePeriod);
	Dmc_param.twr = (uint32_t)ceil(Dmc_param.twr/Dmc_config->DDRClockTimePeriod);

	Dmc_param.trrd = (uint32_t)ceil(Dmc_param.trrd);
	Dmc_param.tmrd = (uint32_t)ceil(Dmc_param.tmrd);
	Dmc_param.twtr = (uint32_t)ceil(Dmc_param.twtr);
	Dmc_param.trtp = (uint32_t)ceil(Dmc_param.trtp);
	Dmc_param.txp = (uint32_t)ceil(Dmc_param.txp);
	Dmc_param.tcke = (uint32_t)ceil(Dmc_param.tcke);
	Dmc_param.datacyc = (uint32_t)ceil(Dmc_param.datacyc);

	Dmc_param.cwl = Dmc_param.cwl - DDR3_CWL_OFFSET;

	eResult = SetDdrCl(Dmc_param.cl,&Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	eResult = SetDdrWrrec(Dmc_param.twr,&Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	eResult = SetMemODT( Dmc_config->MemOdt, &Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	eResult = SetMemDRV( Dmc_config->MemDrv, &Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}


	Dmc_param.zqctl0 = ((Dmc_config->ProAddCmdDrv<<BITP_DMC_DDR_ZQ_CTL0_IMPWRADD) & BITM_DMC_DDR_ZQ_CTL0_IMPWRADD) | ((Dmc_config->ProClkDqsDrv<<BITP_DMC_DDR_ZQ_CTL0_IMPWRDQ) & BITM_DMC_DDR_ZQ_CTL0_IMPWRDQ)  | (((uint32_t)(Dmc_config->ProOdt*2.0f*0.8f)<<BITP_DMC_DDR_ZQ_CTL0_IMPRTT) & BITM_DMC_DDR_ZQ_CTL0_IMPRTT);
	Dmc_param.zqctl1 = 0ul;
	Dmc_param.zqctl2 = 0x70000000ul;

	uint32_t CFG0_BIT_DMC_CFG_SDRSIZE, CFG0_REG_DMC_MR2_VALUE, CFG0_REG_DMC_MR1_VALUE, CFG0_REG_DMC_MR0_VALUE;


/*****************************************DMC_DLLCTL*******************************************/
	/*! DLL Calibration RD Count */
	#define ADI_DMC_PARAM_DLLCOUNT                    240ul

	if(Dmc_config->MemSize == ADI_DDR3_MEM_512Mb)
		CFG0_BIT_DMC_CFG_SDRSIZE = ENUM_DMC_CFG_SDRSIZE512;
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_1Gb)
		CFG0_BIT_DMC_CFG_SDRSIZE = ENUM_DMC_CFG_SDRSIZE1G;
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_2Gb)
		CFG0_BIT_DMC_CFG_SDRSIZE = ENUM_DMC_CFG_SDRSIZE2G;
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_4Gb)
		CFG0_BIT_DMC_CFG_SDRSIZE = ENUM_DMC_CFG_SDRSIZE4G;
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_8Gb)
		CFG0_BIT_DMC_CFG_SDRSIZE = ENUM_DMC_CFG_SDRSIZE8G;

	// Call the Phy and ctrl init functions
	config.ulDDR_DLLCTLCFG = (ENUM_DMC_CFG_IFWID16|ENUM_DMC_CFG_SDRWID16|CFG0_BIT_DMC_CFG_SDRSIZE|ENUM_DMC_CFG_EXTBANK1 | (((ADI_DMC_PARAM_DLLCOUNT<<BITP_DMC_DLLCTL_DLLCALRDCNT) | ((uint32_t)Dmc_param.datacyc<<BITP_DMC_DLLCTL_DATACYC))<<16));

/**************************************** DMC  MR2MR3**********************************************/
	if(Dmc_config->MemTemp == ADI_DDR3_MEM_NOMINAL_TEMP)
		CFG0_REG_DMC_MR2_VALUE  = ((uint32_t)Dmc_param.cwl<<BITP_DMC_MR2_CWL);
	else
		CFG0_REG_DMC_MR2_VALUE = (((uint32_t)Dmc_param.cwl<<BITP_DMC_MR2_CWL)|BITM_DMC_MR2_SRT);

	config.ulDDR_EMR2EMR3 = (((CFG0_REG_DMC_MR2_VALUE)<<16) | BITM_DMC_EMR3_MPR);

/*****************************************DMC_CTL**********************************************/
	/*! enable DDR3 controller */
	#define CFG0_BIT_DMC_CTL_DDR3EN                     1ul

	/*! Constant value, should not be altered*/
	#define CFG0_BIT_DMC_CTL_RDTOWR                     5ul

	if(Dmc_config->DDRSpeedBin > ADI_DDR3_MEM_1600)
	{
		config.ulDDR_CTL = ((CFG0_BIT_DMC_CTL_DDR3EN<<BITP_DMC_CTL_DDR3EN)|BITM_DMC_CTL_INIT|(CFG0_BIT_DMC_CTL_RDTOWR<<BITP_DMC_CTL_RDTOWR)|BITM_DMC_CTL_AL_EN|BITM_DMC_CTL_DDR3_2133|BITM_DMC_CTL_RDECMDDAT);
	}
	else if(Dmc_config->DDRSpeedBin > ADI_DDR3_MEM_1333)
	{
		config.ulDDR_CTL = ((CFG0_BIT_DMC_CTL_DDR3EN<<BITP_DMC_CTL_DDR3EN)|BITM_DMC_CTL_INIT|(CFG0_BIT_DMC_CTL_RDTOWR<<BITP_DMC_CTL_RDTOWR)|BITM_DMC_CTL_AL_EN|BITM_DMC_CTL_RDECMDDAT);
	}
	else
	{
		config.ulDDR_CTL = ((CFG0_BIT_DMC_CTL_DDR3EN<<BITP_DMC_CTL_DDR3EN)|BITM_DMC_CTL_INIT|(CFG0_BIT_DMC_CTL_RDTOWR<<BITP_DMC_CTL_RDTOWR|BITM_DMC_CTL_RDECMDDAT));
	}

/****************************************DMC  MR0**********************************************/

	/*! Constant value, should not be altered*/
	#define CFG0_BIT_DMC_MR0_BLEN                           0ul

	/*! Constant value, should not be altered*/
	#define CFG0_BIT_DMC_MR0_DLLRST                         1ul

	CFG0_REG_DMC_MR0_VALUE = ((CFG0_BIT_DMC_MR0_BLEN<<BITP_DMC_MR_BLEN)|((uint32_t)Dmc_param.cl0<<BITP_DMC_MR_CL0)|((uint32_t)Dmc_param.cl<<BITP_DMC_MR_CL)|(CFG0_BIT_DMC_MR0_DLLRST<<BITP_DMC_MR_DLLRST)|((uint32_t)Dmc_param.wrrec<<BITP_DMC_MR_WRRECOV));

/****************************************DMC  MR1**********************************************/
	/*! Constant value, should not be altered*/
	#define CFG0_BIT_DMC_MR1_DLLEN                            0ul
	#define CFG0_BIT_DMC_MR1_TDQS                             0ul
	#define CFG0_BIT_DMC_MR1_QOFF                             0ul
	CFG0_REG_DMC_MR1_VALUE = ((CFG0_BIT_DMC_MR1_DLLEN<<BITP_DMC_MR1_DLLEN)|((uint32_t)Dmc_param.dic0<<BITP_DMC_MR1_DIC0)|((uint32_t)Dmc_param.rtt0<<BITP_DMC_MR1_RTT0)|((uint32_t)Dmc_param.al<<BITP_DMC_MR1_AL)|((uint32_t)Dmc_param.dic1<<BITP_DMC_MR1_DIC1)|((uint32_t)Dmc_param.rtt1<<BITP_DMC_MR1_RTT1)|((uint32_t)Dmc_param.rtt2<<BITP_DMC_MR1_RTT2)|(CFG0_BIT_DMC_MR1_TDQS<<BITP_DMC_MR1_TDQS)|(CFG0_BIT_DMC_MR1_QOFF<<BITP_DMC_MR1_QOFF)|(BITM_DMC_EMR1_WL));

	config.ulDDR_MREMR1 = (CFG0_REG_DMC_MR1_VALUE|(CFG0_REG_DMC_MR0_VALUE<<16));

/*****************************************DMC_TR0**********************************************/
	config.ulDDR_TR0 = (((uint32_t)Dmc_param.trcd<<BITP_DMC_TR0_TRCD)|((uint32_t)Dmc_param.twtr<<BITP_DMC_TR0_TWTR)|((uint32_t)Dmc_param.trp<<BITP_DMC_TR0_TRP)|((uint32_t)Dmc_param.tras<<BITP_DMC_TR0_TRAS)|((uint32_t)Dmc_param.trc<<BITP_DMC_TR0_TRC)|((uint32_t)Dmc_param.tmrd<<BITP_DMC_TR0_TMRD));

/*****************************************DMC_TR1**********************************************/
	config.ulDDR_TR1 = (((uint32_t)Dmc_param.trefi<<BITP_DMC_TR1_TREF)|((uint32_t)Dmc_param.trfc<<BITP_DMC_TR1_TRFC)|((uint32_t)Dmc_param.trrd<<BITP_DMC_TR1_TRRD));

/*****************************************DMC_TR2**********************************************/
	config.ulDDR_TR2 = (((uint32_t)Dmc_param.tfaw<<BITP_DMC_TR2_TFAW)|((uint32_t)Dmc_param.trtp<<BITP_DMC_TR2_TRTP)|((uint32_t)Dmc_param.txp<<BITP_DMC_TR2_TXP)|((uint32_t)Dmc_param.tcke<<BITP_DMC_TR2_TCKE));

/*****************************************DMC_ZQCTL0**********************************************/
	config.ulDDR_ZQCTL0 = Dmc_param.zqctl0;

/*****************************************DMC_ZQCTL1**********************************************/
	config.ulDDR_ZQCTL1 = Dmc_param.zqctl1;

/*****************************************DMC_ZQCTL2**********************************************/
	config.ulDDR_ZQCTL2 = Dmc_param.zqctl2;


	/* Initialize DMC PHY registers */
    adi_dmc_phy_calibration(&config);

    /* Initialize DMC Controller */
    if(adi_dmc_ctrl_init(&config) != ADI_DMC_SUCCESS)
    {
		return ADI_DMC_FAILURE;
    }
    else
    {
    	return ADI_DMC_SUCCESS;
    }

}


ADI_DMC_RESULT adi_printDMCconfig(ADI_DMC_PARAM_LIST* Dmc_config)
{
	ADI_DMC_PARAM_LIST_INT Dmc_param;
	ADI_DMC_RESULT eResult= ADI_DMC_SUCCESS;

	Dmc_param.trcd = speedbinTimingTable[ADI_DDR3_MEM_TRCD_TRP][Dmc_config->DDRSpeedBin];
	Dmc_param.trp = speedbinTimingTable[ADI_DDR3_MEM_TRCD_TRP][Dmc_config->DDRSpeedBin];
	Dmc_param.tras = speedbinTimingTable[ADI_DDR3_MEM_TRAS][Dmc_config->DDRSpeedBin];
	Dmc_param.trc = speedbinTimingTable[ADI_DDR3_MEM_TRC][Dmc_config->DDRSpeedBin];
	Dmc_param.tmrd = speedbinTimingTable[ADI_DDR3_MEM_TMRD][Dmc_config->DDRSpeedBin];
	Dmc_param.trefi = trefiTable[Dmc_config->MemTemp][Dmc_config->MemSize];
	Dmc_param.trfc = trfcTable[Dmc_config->MemSize];
	Dmc_param.tfaw = speedbinTimingTable[ADI_DDR3_MEM_TFAW][Dmc_config->DDRSpeedBin];
	Dmc_param.twr = speedbinTimingTable[ADI_DDR3_MEM_TWR][Dmc_config->DDRSpeedBin];
	Dmc_param.cl = speedbinTimingTable[ADI_DDR3_MEM_CL][Dmc_config->DDRSpeedBin];
	Dmc_param.cwl = speedbinTimingTable[ADI_DDR3_MEM_CWL][Dmc_config->DDRSpeedBin];
	Dmc_param.datacyc = 15/Dmc_config->DDRClockTimePeriod;
	Dmc_param.al = ADI_DDR3_AL_0;

	Dmc_param.twtr = getTwtr(Dmc_config->DDRClockTimePeriod);
	Dmc_param.trrd = getTrrd(Dmc_config->DDRClockTimePeriod,Dmc_config->DDRSpeedBin);
	Dmc_param.trtp = getTrtp(Dmc_config->DDRClockTimePeriod);
	Dmc_param.tcke = getTcke(Dmc_config->DDRClockTimePeriod,Dmc_config->DDRSpeedBin);
	Dmc_param.txp = getTxp(Dmc_config->DDRClockTimePeriod,Dmc_config->DDRSpeedBin);


	Dmc_param.trcd = (uint32_t)ceil(Dmc_param.trcd/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trp = (uint32_t)ceil(Dmc_param.trp/Dmc_config->DDRClockTimePeriod);
	Dmc_param.tras = (uint32_t)ceil(Dmc_param.tras/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trc = (uint32_t)ceil(Dmc_param.trc/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trefi = (uint32_t)ceil(Dmc_param.trefi/Dmc_config->DDRClockTimePeriod);
	Dmc_param.trfc = (uint32_t)ceil(Dmc_param.trfc/Dmc_config->DDRClockTimePeriod);
	Dmc_param.tfaw = (uint32_t)ceil(Dmc_param.tfaw/Dmc_config->DDRClockTimePeriod);
	Dmc_param.twr = (uint32_t)ceil(Dmc_param.twr/Dmc_config->DDRClockTimePeriod);

	Dmc_param.trrd = (uint32_t)ceil(Dmc_param.trrd);
	Dmc_param.tmrd = (uint32_t)ceil(Dmc_param.tmrd);
	Dmc_param.twtr = (uint32_t)ceil(Dmc_param.twtr);
	Dmc_param.trtp = (uint32_t)ceil(Dmc_param.trtp);
	Dmc_param.txp = (uint32_t)ceil(Dmc_param.txp);
	Dmc_param.tcke = (uint32_t)ceil(Dmc_param.tcke);
	Dmc_param.datacyc = (uint32_t)ceil(Dmc_param.datacyc);

	Dmc_param.cwl = Dmc_param.cwl - DDR3_CWL_OFFSET;

	eResult = SetDdrCl(Dmc_param.cl,&Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	eResult = SetDdrWrrec(Dmc_param.twr,&Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	eResult = SetMemODT( Dmc_config->MemOdt, &Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	eResult = SetMemDRV( Dmc_config->MemDrv, &Dmc_param);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}


	Dmc_param.zqctl0 = ((Dmc_config->ProAddCmdDrv<<BITP_DMC_DDR_ZQ_CTL0_IMPWRADD) & BITM_DMC_DDR_ZQ_CTL0_IMPWRADD) | ((Dmc_config->ProClkDqsDrv<<BITP_DMC_DDR_ZQ_CTL0_IMPWRDQ) & BITM_DMC_DDR_ZQ_CTL0_IMPWRDQ)  | (((uint32_t)(Dmc_config->ProOdt*2.0f*0.8f)<<BITP_DMC_DDR_ZQ_CTL0_IMPRTT) & BITM_DMC_DDR_ZQ_CTL0_IMPRTT);
	Dmc_param.zqctl1 = 0ul;
	Dmc_param.zqctl2 = 0x70000000ul;


	printf("\n CFG0_BIT_DMC_CTL_DDR3EN			1ul");

	printf("\n CFG0_BIT_DMC_CTL_RDTOWR			5ul");

	if(Dmc_config->DDRSpeedBin > ADI_DDR3_MEM_1600)
	{
		printf("\n CFG0_REG_DMC_CTL_VALUE			((CFG0_BIT_DMC_CTL_DDR3EN<<BITP_DMC_CTL_DDR3EN)|BITM_DMC_CTL_INIT|(CFG0_BIT_DMC_CTL_RDTOWR<<BITP_DMC_CTL_RDTOWR)|BITM_DMC_CTL_AL_EN|BITM_DMC_CTL_DDR3_2133) \n");
	}
	else if(Dmc_config->DDRSpeedBin > ADI_DDR3_MEM_1333)
	{
		printf("\n CFG0_REG_DMC_CTL_VALUE			((CFG0_BIT_DMC_CTL_DDR3EN<<BITP_DMC_CTL_DDR3EN)|BITM_DMC_CTL_INIT|(CFG0_BIT_DMC_CTL_RDTOWR<<BITP_DMC_CTL_RDTOWR)|BITM_DMC_CTL_AL_EN) \n");
	}
	else
	{
		printf("\n CFG0_REG_DMC_CTL_VALUE			((CFG0_BIT_DMC_CTL_DDR3EN<<BITP_DMC_CTL_DDR3EN)|BITM_DMC_CTL_INIT|(CFG0_BIT_DMC_CTL_RDTOWR<<BITP_DMC_CTL_RDTOWR)) \n");
	}

	if(Dmc_config->MemSize == ADI_DDR3_MEM_512Mb)
		printf("\n #define CFG0_BIT_DMC_CFG_SDRSIZE                    (ENUM_DMC_CFG_SDRSIZE512)");
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_1Gb)
		printf("\n #define CFG0_BIT_DMC_CFG_SDRSIZE                    (ENUM_DMC_CFG_SDRSIZE1G)");
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_2Gb)
		printf("\n #define CFG0_BIT_DMC_CFG_SDRSIZE                    (ENUM_DMC_CFG_SDRSIZE2G)");
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_4Gb)
		printf("\n #define CFG0_BIT_DMC_CFG_SDRSIZE                    (ENUM_DMC_CFG_SDRSIZE4G)");
	else if(Dmc_config->MemSize == ADI_DDR3_MEM_8Gb)
		printf("\n #define CFG0_BIT_DMC_CFG_SDRSIZE                    (ENUM_DMC_CFG_SDRSIZE8G)");

	printf("\n\n #define CFG0_REG_DMC_CFG_VALUE \\ \n  (ENUM_DMC_CFG_IFWID16     | /* Interface Width - always 16-bit */ \\ \n   ENUM_DMC_CFG_SDRWID16    | /* SDRAM Width - always 16-bit */ \\ \n   CFG0_BIT_DMC_CFG_SDRSIZE | /* SDRAM Size */ \\ \n   ENUM_DMC_CFG_EXTBANK1)     /* External Banks - always 1 bank */ ");

	printf("\n\n ADI_DMC_PARAM_DLLCOUNT			240ul");

	printf("\n\n CFG0_REG_DMC_DATACYC			%dul",(uint32_t)Dmc_param.datacyc);

	printf("\n\n #define CFG0_REG_DMC_DLLCTL_VALUE \\ \n  ((ADI_DMC_PARAM_DLLCOUNT<<BITP_DMC_DLLCTL_DLLCALRDCNT) | \\ \n   (CFG0_REG_DMC_DATACYC<<BITP_DMC_DLLCTL_DATACYC))");

	printf("\n\n CFG0_BIT_DMC_TR0_TRCD			%dul",(uint32_t)Dmc_param.trcd);
	printf("\n CFG0_BIT_DMC_TR0_TWTR			%dul",(uint32_t)Dmc_param.twtr);
	printf("\n CFG0_BIT_DMC_TR0_TRP			    %dul",(uint32_t)Dmc_param.trp);
	printf("\n CFG0_BIT_DMC_TR0_TRAS			%dul",(uint32_t)Dmc_param.tras);
	printf("\n CFG0_BIT_DMC_TR0_TRC				%dul",(uint32_t)Dmc_param.trc);
	printf("\n CFG0_BIT_DMC_TR0_TMRD			%dul",(uint32_t)Dmc_param.tmrd);
	printf("\n\n #define CFG0_REG_DMC_TR0_VALUE      ((CFG0_BIT_DMC_TR0_TRCD<<BITP_DMC_TR0_TRCD)|(CFG0_BIT_DMC_TR0_TWTR<<BITP_DMC_TR0_TWTR)|(CFG0_BIT_DMC_TR0_TRP<<BITP_DMC_TR0_TRP)|(CFG0_BIT_DMC_TR0_TRAS<<BITP_DMC_TR0_TRAS)|(CFG0_BIT_DMC_TR0_TRC<<BITP_DMC_TR0_TRC)|(CFG0_BIT_DMC_TR0_TMRD<<BITP_DMC_TR0_TMRD)) ");


	printf("\n\n CFG0_BIT_DMC_TR1_TREF			%dul",(uint32_t)Dmc_param.trefi);
	printf("\n CFG0_BIT_DMC_TR1_TRFC			%dul",(uint32_t)Dmc_param.trfc);
	printf("\n CFG0_BIT_DMC_TR1_TRRD			%dul",(uint32_t)Dmc_param.trrd);
	printf("\n\n#define CFG0_REG_DMC_TR1_VALUE      ((CFG0_BIT_DMC_TR1_TREF<<BITP_DMC_TR1_TREF)|(CFG0_BIT_DMC_TR1_TRFC<<BITP_DMC_TR1_TRFC)|(CFG0_BIT_DMC_TR1_TRRD<<BITP_DMC_TR1_TRRD))");

	printf("\n\n CFG0_BIT_DMC_TR2_TFAW			%dul",(uint32_t)Dmc_param.tfaw);
	printf("\n CFG0_BIT_DMC_TR2_TRTP			%dul",(uint32_t)Dmc_param.trtp);
	printf("\n CFG0_BIT_DMC_TR2_TXP				%dul",(uint32_t)Dmc_param.txp);
	printf("\n CFG0_BIT_DMC_TR2_TCKE			%dul",(uint32_t)Dmc_param.tcke);
	printf("\n\n #define CFG0_REG_DMC_TR2_VALUE      ((CFG0_BIT_DMC_TR2_TFAW<<BITP_DMC_TR2_TFAW)|(CFG0_BIT_DMC_TR2_TRTP<<BITP_DMC_TR2_TRTP)|(CFG0_BIT_DMC_TR2_TWR<<BITP_DMC_TR2_TWR)|(CFG0_BIT_DMC_TR2_TXP<<BITP_DMC_TR2_TXP)|(CFG0_BIT_DMC_TR2_TCKE<<BITP_DMC_TR2_TCKE)) ");

	printf("\n\n CFG0_REG_DDR_DLLCTLCFG \\			");
	printf("\n (CFG0_REG_DMC_CFG_VALUE |    \\		");
	printf("\n (CFG0_REG_DMC_DLLCTL_VALUE<<16))		");

	printf("\n\n CFG0_BIT_DMC_MR0_BLEN			0ul");
	printf("\n CFG0_BIT_DMC_MR0_CL0				%dul",(uint32_t)Dmc_param.cl0);
	printf("\n CFG0_BIT_DMC_MR0_CL				%dul",(uint32_t)Dmc_param.cl);
	printf("\n CFG0_BIT_DMC_MR0_DLLRST			1ul");
	printf("\n CFG0_BIT_DMC_MR0_WRRECOV			%dul",(uint32_t)Dmc_param.wrrec);

	printf("\n\n #define CFG0_REG_DMC_MR0_VALUE             ((CFG0_BIT_DMC_MR0_BLEN<<BITP_DMC_MR_BLEN)|(CFG0_BIT_DMC_MR0_CL0<<BITP_DMC_MR_CL0)|(CFG0_BIT_DMC_MR0_CL<<BITP_DMC_MR_CL)|(CFG0_BIT_DMC_MR0_DLLRST<<BITP_DMC_MR_DLLRST)|(CFG0_BIT_DMC_MR0_WRRECOV<<BITP_DMC_MR_WRRECOV))");

	printf("\n\n CFG0_BIT_DMC_MR1_DLLEN			0ul");
	printf("\n CFG0_BIT_DMC_MR1_DIC0			%dul",(uint32_t)Dmc_param.dic0);
	printf("\n CFG0_BIT_DMC_MR1_RTT0			%dul",(uint32_t)Dmc_param.rtt0);
	printf("\n CFG0_BIT_DMC_MR1_AL 			    %dul",(uint32_t)Dmc_param.al);
	printf("\n CFG0_BIT_DMC_MR1_DIC1 			%dul",(uint32_t)Dmc_param.dic1);
	printf("\n CFG0_BIT_DMC_MR1_RTT1 			%dul",(uint32_t)Dmc_param.rtt1);
	printf("\n CFG0_BIT_DMC_MR1_RTT2 			%dul",(uint32_t)Dmc_param.rtt2);
	printf("\n CFG0_BIT_DMC_MR1_TDQS			0ul");
	printf("\n CFG0_BIT_DMC_MR1_QOFF			0ul");
	printf("\n\n #define CFG0_REG_DMC_MR1_VALUE ((CFG0_BIT_DMC_MR1_DLLEN<<BITP_DMC_MR1_DLLEN)|(CFG0_BIT_DMC_MR1_DIC0<<BITP_DMC_MR1_DIC0)|(CFG0_BIT_DMC_MR1_RTT0<<BITP_DMC_MR1_RTT0)|(CFG0_BIT_DMC_MR1_AL<<BITP_DMC_MR1_AL)|(CFG0_BIT_DMC_MR1_DIC1<<BITP_DMC_MR1_DIC1)|(CFG0_BIT_DMC_MR1_RTT1<<BITP_DMC_MR1_RTT1)|(CFG0_BIT_DMC_MR1_RTT2<<BITP_DMC_MR1_RTT2)|(CFG0_BIT_DMC_MR1_TDQS<<BITP_DMC_MR1_TDQS)|(CFG0_BIT_DMC_MR1_QOFF<<BITP_DMC_MR1_QOFF)|(BITM_DMC_EMR1_WL))");

	printf("\n\n CFG0_BIT_DMC_MR2_CWL 			%dul",(uint32_t)Dmc_param.cwl);

	if(Dmc_config->MemTemp == ADI_DDR3_MEM_NOMINAL_TEMP)
	{
		printf("\n\n #define CFG0_REG_DMC_MR2_VALUE (CFG0_BIT_DMC_MR2_CWL<<BITP_DMC_MR2_CWL) ");
	}
	else
	{
		printf("\n\n #define CFG0_REG_DMC_MR2_VALUE ((CFG0_BIT_DMC_MR2_CWL<<BITP_DMC_MR2_CWL)|BITM_DMC_MR2_SRT) ");
	}

	printf("\n\n#define CFG0_REG_DMC_MR2MR3      (((CFG0_REG_DMC_MR2_VALUE)<<16)|BITM_DMC_EMR3_MPR) ");

	printf("\n\n#define CFG0_REG_DMC_MRMR1        (CFG0_REG_DMC_MR1_VALUE|(CFG0_REG_DMC_MR0_VALUE<<16)) ");

	printf("\n\n/* ADI_DMC_PARAM_LIST_INT structure */");

	printf("\n\n    static ADI_DMC_CONFIG config = ");
	printf("\n    { ");
	printf("\n    CFG0_REG_DDR_DLLCTLCFG,");
	printf("\n    CFG0_REG_DMC_MR2MR3,");
	printf("\n    CFG0_REG_DMC_CTL_VALUE,");
	printf("\n    CFG0_REG_DMC_MRMR1,");
	printf("\n    CFG0_REG_DMC_TR0_VALUE,");
	printf("\n    CFG0_REG_DMC_TR1_VALUE,");
	printf("\n    CFG0_REG_DMC_TR2_VALUE,");
	printf("\n    0x%Xul",(uint32_t)Dmc_param.zqctl0);
	printf("\n    0x%Xul",(uint32_t)Dmc_param.zqctl1);
	printf("\n    0x%Xul",(uint32_t)Dmc_param.zqctl2);
	printf("\n    } ");

	return eResult;
}
