
#ifndef ADI_DMC_CONFIG_GENERATOR_H_
#define ADI_DMC_CONFIG_GENERATOR_H_

#include<stdio.h>
#include<stdint.h>
#include <adi_types.h>
#include "adi_dmc.h"

#define DDR3_CWL_OFFSET    5u

float32_t cclk_dclk_r(void);


typedef enum{
	/* used for array indexing , don't change the numbers*/
	ADI_DDR3_MEM_512Mb=0,
	ADI_DDR3_MEM_1Gb,
	ADI_DDR3_MEM_2Gb,
	ADI_DDR3_MEM_4Gb,
	ADI_DDR3_MEM_8Gb,
	NUM_MEM_SIZE
}ADI_DDR3_MEM_SIZE;

typedef enum{
	/* used for array indexing , don't change the numbers*/
	ADI_DDR3_MEM_NOMINAL_TEMP=0,
	ADI_DDR3_MEM_EXTENDED_TEMP,
	NUM_TEMP
}ADI_DDR3_MEM_TEMP_RANGE;

typedef enum{
	/* used for array indexing , don't change the numbers*/
	ADI_DDR3_MEM_TRCD_TRP=0,
	ADI_DDR3_MEM_TRC,
	ADI_DDR3_MEM_TRAS,
	ADI_DDR3_MEM_CL,
	ADI_DDR3_MEM_CWL,
	ADI_DDR3_MEM_TFAW,
	ADI_DDR3_MEM_TWR,
	ADI_DDR3_MEM_TMRD,
	NUM_SPEEDBIN_TIMING
}ADI_DDR3_MEM_SPEEDBIN_TIMING;
;

typedef enum{
	/* used for array indexing , don't change the numbers*/
	ADI_DDR3_MEM_800=0,
	ADI_DDR3_MEM_1066,
	ADI_DDR3_MEM_1333,
	ADI_DDR3_MEM_1600,
	ADI_DDR3_MEM_1866,
	ADI_DDR3_MEM_2133,
	NUM_SPEED_BIN
}ADI_DDR3_MEM_SPEED_BINS;

typedef enum
{
	/*! Success */
	ADI_DMC_SUCCESS = 0u,
	/*! Generic failure. */
	ADI_DMC_FAILURE,
} ADI_DMC_RESULT;

typedef enum{
	ADI_DDR3_ODT_60=1u,
	ADI_DDR3_ODT_120,
	ADI_DDR3_ODT_40,
	ADI_DDR3_ODT_20,
	ADI_DDR3_ODT_30
}ADI_DDR3_MEMORY_ODT;

typedef enum{
	ADI_DDR3_DRV_40=0u,
	ADI_DDR3_DRV_34
}ADI_DDR3_MEMORY_DRV;

typedef enum{
	ADI_DDR3_AL_0=0u,
	ADI_DDR3_AL_1,
	ADI_DDR3_AL_2
}ADI_DDR3_MEMORY_AL;

typedef enum{
	ADI_DDR3_FULL_ARR=0u,
	ADI_DDR3_HARR0,
	ADI_DDR3_QARR0,
	ADI_DDR3_1_8_ARR0,
	ADI_DDR3_3_4_ARR,
	ADI_DDR3_HARR1,
	ADI_DDR3_QARR1,
	ADI_DDR3_1_8_ARR1
}ADI_DDR3_MEMORY_PASR;

typedef enum{
	ADI_DMC_PRECBANK0=100u,
	ADI_DMC_PRECBANK1,
	ADI_DMC_PRECBANK2,
	ADI_DMC_PRECBANK3,
	ADI_DMC_PRECBANK4,
	ADI_DMC_PRECBANK5,
	ADI_DMC_PRECBANK6,
	ADI_DMC_PRECBANK7
}ADI_DMC_PRECHARGE_BANK;


/* timing parameters in terms of number of DDR clock cycles  */
typedef struct _ADI_DMC_PARAM_LIST_INT_
{
	float trcd;
	float twtr;
	float trp;
	float tras;
	float trc;
	float tmrd;
	float trefi;
	float trfc;
	float trrd;
	float tfaw;
	float twr;
	float trtp;
	float wrrec;
	float txp;
	float tcke;
	float cl;
	float cl0;
	float cwl;
	float datacyc;
	ADI_DDR3_MEMORY_AL al;
	uint32_t rtt0;
	uint32_t rtt1;
	uint32_t rtt2;
	uint32_t dic0;
	uint32_t dic1;
	uint32_t zqctl0;
	uint32_t zqctl1;
	uint32_t zqctl2;
} ADI_DMC_PARAM_LIST_INT;

/* Parameter list to be configured by user   */
typedef struct _ADI_DMC_PARAM_LIST_
{
	float DDRClockTimePeriod;
	float CoreClockTimePeriod;
	ADI_DDR3_MEM_SPEED_BINS DDRSpeedBin;
	uint32_t ProClkDqsDrv;
	uint32_t ProAddCmdDrv;
	uint32_t ProOdt;
	ADI_DDR3_MEM_SIZE MemSize;
	ADI_DDR3_MEM_TEMP_RANGE MemTemp;
	ADI_DDR3_MEMORY_ODT MemOdt;
	ADI_DDR3_MEMORY_DRV MemDrv;
} ADI_DMC_PARAM_LIST;

/* structure which holds DMC register values */
typedef struct
{

  uint32_t ulDDR_DLLCTLCFG;               /*!< Content of DDR DLLCTL and DMC_CFG register     */
  uint32_t ulDDR_EMR2EMR3;                /*!< Content of the DDR EMR2 and EMR3 Register      */
  uint32_t ulDDR_CTL;                     /*!< Content of the DDR Control                   */
  uint32_t ulDDR_MREMR1;                  /*!< Content of the DDR MR and EMR1 Register      */
  uint32_t ulDDR_TR0;                     /*!< Content of the DDR Timing Register      */
  uint32_t ulDDR_TR1;                     /*!< Content of the DDR Timing Register      */
  uint32_t ulDDR_TR2;                     /*!< Content of the DDR Timing Register      */
  uint32_t ulDDR_ZQCTL0;                  /*!< Content of ZQCTL0 register */
  uint32_t ulDDR_ZQCTL1;                  /*!< Content of ZQCTL1 register */
  uint32_t ulDDR_ZQCTL2;                  /*!< Content of ZQCTL2 register */

}ADI_DMC_CONFIG;

#ifdef __cplusplus
extern "C" {
#endif

/* reset dmc lanes */
void adi_dmc_lane_reset(bool reset);
/* enter/exit self refresh mode */
void adi_dmc_self_refresh(bool enter);
/* program dmc controller timing registers */
ADI_DMC_RESULT adi_dmc_ctrl_init(ADI_DMC_CONFIG *pConfig);
/* trigger Zq calibration*/
void adi_dmc_phy_calibration(ADI_DMC_CONFIG *pConfig);

ADI_DMC_RESULT adi_DMCinit(ADI_DMC_PARAM_LIST* Dmc_config);
ADI_DMC_RESULT adi_printDMCconfig(ADI_DMC_PARAM_LIST* Dmc_config);


#ifdef __cplusplus
}
#endif


#endif /* ADI_DDR_H_ */
