/*****************************************************************************
 * Preload_DMCconfigGenerator_Core1.c
 *****************************************************************************/

#include "main.h"

#include "adi_dmc_config_generator.h"
#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"
#include "ddr_sweep.h"
#include "adi_dmc.h"
#include "adi_pwr_SC59x_config.h"

/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

ADI_DMC_PARAM_LIST Dmc_config;

ADI_DMC_RESULT eResult;

float32_t cclkdclk_ratio;

int main(int argc, char *argv[])
{
	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */
	adi_initComponents();

	ADI_DMC_RESULT eResult;

	Dmc_config.DDRSpeedBin = ADI_DDR3_MEM_1600;		/* Choose the Speedbin based on the DDR clock frequency */
	Dmc_config.DDRClockTimePeriod = 1.25f;			/* Time period or tck of DDR clock */
	Dmc_config.CoreClockTimePeriod = 1.00f;			/* Time period of Core clock */
	Dmc_config.ProAddCmdDrv = 100;					/* Processor's DriveStrength of Address and command */
	Dmc_config.ProClkDqsDrv = 90;					/* Processor's DriveStrength of DQ, DQS, DM and clock */
	Dmc_config.ProOdt = 75;							/* Processor's ODT of DQ, DQS, DM */
	Dmc_config.MemSize = ADI_DDR3_MEM_4Gb;			/* size of DDR memory */
	Dmc_config.MemTemp = ADI_DDR3_MEM_NOMINAL_TEMP; /* Operating temperature range of DDR memory */
	Dmc_config.MemDrv = ADI_DDR3_DRV_40;			/* DDR memory's DriveStength */
	Dmc_config.MemOdt = ADI_DDR3_ODT_120;			/* DDR memory's ODT */

	cclkdclk_ratio =  (Dmc_config.DDRClockTimePeriod/ Dmc_config.CoreClockTimePeriod);

	/* The generated DMC configurations are valid only for ADSP-2159x/SC59x SHARC+ processors
	 * refer to datasheet for valid Dclk frequencies supported
	 * The configurations are based on JEDEC specification, cross check with the DDR memory datasheet */

#ifdef Print_DMC_Config
	/* Prints the Macros required to initialize ADI_DMC_CONFIG structure */
	eResult =  adi_printDMCconfig(&Dmc_config);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}
#endif

#ifdef Initialize_DMC_Basic

    /* Set DMC Lane Reset */
    adi_dmc_lane_reset(true);

    /* Initialize CGU and CDU */
    if((uint32_t)adi_pwr_cfg0_init() != 0)
	{
		return ADI_DMC_FAILURE;
	}

    /* Clear DMC Lane Reset */
    adi_dmc_lane_reset(false);

    /* Initialize DMC */
	eResult =  adi_DMCinit(&Dmc_config);
	if(eResult != ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	if(Memory_Sweep_Test(MEM_START,BYTE_COUNT)>0)
	{
		printf("\n DDR Sweep test fail ");
	}
	else
	{
		printf("\n DDR Sweep test pass ");
	}
#endif

#ifdef Initialize_DMC_Advanced

	cclkdclk_ratio = 1.25;

    /* Set DMC Lane Reset */
    adi_dmc_lane_reset(true);

    /* Initialize CGU and CDU */
    if((uint32_t)adi_pwr_cfg0_init() != 0)
	{
		return ADI_DMC_FAILURE;
	}

    /* Clear DMC Lane Reset */
    adi_dmc_lane_reset(false);

    // add macros here in a header file and that can be used here
    static ADI_DMC_CONFIG config =
    {
    		CFG0_REG_DDR_DLLCTLCFG, 			/* ulDDR_DLLCTLCFG */
			CFG0_REG_DMC_MR2MR3, 				/* ulDDR_EMR2EMR3 */
			CFG0_REG_DMC_CTL_VALUE, 			/* ulDDR_CTL */
			CFG0_REG_DMC_MRMR1, 				/* ulDDR_MREMR1 */
			CFG0_REG_DMC_TR0_VALUE, 			/* ulDDR_TR0 */
			CFG0_REG_DMC_TR1_VALUE, 			/* ulDDR_TR1 */
			CFG0_REG_DMC_TR2_VALUE, 			/* ulDDR_TR2 */
			CFG0_REG_DMC_ZQCTL_0, 				/* ulDDR_ZQCTL0 */
			CFG0_REG_DMC_ZQCTL_1, 				/* ulDDR_ZQCTL1 */
			CFG0_REG_DMC_ZQCTL_2				/* ulDDR_ZQCTL2 */
    };

    /* Initialize DMC PHY registers */
    adi_dmc_phy_calibration(&config);

    /* Initialize DMC Controller */
    if(adi_dmc_ctrl_init(&config) != ADI_DMC_SUCCESS)
    {
		return ADI_DMC_FAILURE;
    }

	if(Memory_Sweep_Test(MEM_START,BYTE_COUNT)>0)
	{
		printf("\n DDR Sweep test fail ");
	}
	else
	{
		printf("\n DDR Sweep test pass ");
	}
#endif

	return 0;
}

