/*
 * Copyright (c) 2018-2022 Analog Devices, Inc. All Rights Reserved.
 * This software is proprietary to Analog Devices, Inc. and its licensors.
 */

#include "DMCReInit.h"
#include "adi_dmc.h"
#include "adi_pwr_SC59x_config.h"

__attribute__((section(".l2_cached_data"))) float32_t cclkdclk_ratio;

ADI_DMC_RESULT DMCReInit(void)
{
	cclkdclk_ratio = 1.8;

	/* Enter DDR Self-Refresh mode */
	adi_dmc_self_refresh(true);

    /* Set DMC Lane Reset */
    adi_dmc_lane_reset(true);

    /* Initialize CGU and CDU and configure DDR clock source to 667MHz */
    if((uint32_t)adi_pwr_cfg0_init() != 0)
	{
		return ADI_DMC_FAILURE;
	}

    /* Clear DMC Lane Reset */
    adi_dmc_lane_reset(false);

	/* Exit DDR Self-Refresh mode */
	adi_dmc_self_refresh(false);

	__attribute__((section(".l2_cached_data"))) static ADI_DMC_CONFIG config =
    {
    		CFG0_REG_DDR_DLLCTLCFG, 			/* ulDDR_DLLCTLCFG */
			CFG0_REG_DMC_MR2MR3, 				/* ulDDR_EMR2EMR3 */
			CFG0_REG_DMC_CTL_VALUE, 			/* ulDDR_CTL */
			CFG0_REG_DMC_MRMR1, 				/* ulDDR_MREMR1 */
			CFG0_REG_DMC_TR0_VALUE, 			/* ulDDR_TR0 */
			CFG0_REG_DMC_TR1_VALUE, 			/* ulDDR_TR1 */
			CFG0_REG_DMC_TR2_VALUE, 			/* ulDDR_TR2 */
			CFG0_REG_DMC_ZQCTL_0, 				/* ulDDR_ZQCTL0 */
			CFG0_REG_DMC_ZQCTL_1, 				/* ulDDR_ZQCTL1 */
			CFG0_REG_DMC_ZQCTL_2				/* ulDDR_ZQCTL2 */
    };

    /* Re-Initialize DMC */

    /* Initialize DMC PHY registers */
    adi_dmc_phy_calibration(&config);

    /* Initialize DMC Controller */
    if(adi_dmc_ctrl_init(&config) != ADI_DMC_SUCCESS)
    {
		return ADI_DMC_FAILURE;
    }

    return ADI_DMC_SUCCESS;
}


