/*
 * Copyright (c) 2020-2022 Analog Devices, Inc. All Rights Reserved.
 * This software is proprietary to Analog Devices, Inc. and its licensors.
 */

/*!
* @file      adi_pwr_SC59x_config.h
*
* @brief     Header file for Power Service Configuration.
*
* @details
*
*/

#ifndef INC_ADI_PWR_SC59X_CONFIG_H_
#define INC_ADI_PWR_SC59X_CONFIG_H_

#include <adi_types.h>

#if defined(__ADSPSHARC__)
  #include <adi_types.h>
  #include <time.h>
#elif defined __ADSPARM__
  #define DO_CYCLE_COUNT
  #include <cycle_count.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

__attribute__((section(".l2_cached_code"))) uint32_t adi_pwr_cfg0_init(void) ;
__attribute__((section(".l2_cached_code"))) void adi_configDCLK_1(uint32_t Msel, uint32_t Dsel) ;



 #if defined(__ADSPSC598_FAMILY__)
 float32_t cclk_clkin_r(void);
 #endif

 /* delay function */
 #if defined __ADSPARM__
 static __inline__ void Cclkdelay(uint32_t delay)
 {

   CCNTR_INIT;
   CCNTR_START;
   /* Wait for the appropriate number of cycles */
   while (CCNTR_READ < delay) {
     /* Wait */
   }
   CCNTR_STOP;
 }
 #elif defined __ADSPSHARC__
 #pragma inline
 static void Cclkdelay(uint32_t delay)
 {
   /* Although the clock() function returns a signed value, startCycleCount
    * must be unsigned so that the comparison in the 'while' loop is valid if
    * the emuclk register value wraps around.
    */
   uint32_t startCycleCount;

   /*
    * Wait for the appropriate number of cycles. On SHARC processors, the
    * clock() function in the run-time library will return the cycle count in
    * the emuclk register.
    */
   startCycleCount = clock();
   while (((uint32_t)(clock()) - startCycleCount) < delay) {
     /* Wait */
   }
 }
 #endif


#ifdef __cplusplus
}
#endif

#endif /* INC_ADI_PWR_SC59X_CONFIG_H_ */
