/*****************************************************************************
 * Preload_DMCconfigGenerator_Core1.c
 *****************************************************************************/

#include "main.h"

#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"
#include "DMCReInit.h"
#include <adi_osal.h>


/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";


uint32_t check_array[BYTE_COUNT/4];

#if defined(__ADSPCORTEXA5__)

#include <runtime/cache/adi_cache.h>
#include <stdint.h>

void disable_mmu(void);

/* Set the global control variable adi_cache_gEnable to 0 so the cache runtime
 * support does not enable either of the L1 or L2 caches. This is done so that
 * emulator doesn't attempt to invalidate caches while loading the application
 * as doing so increases load times considerably.
 */
uint32_t adi_cache_gEnable = 0uL;

/* Define a stub adi_cache_Init function to replace the default one
 * to ensure caches aren't enabled and to avoid cache support functions
 * being linked in unnecessarily.
 */
ADI_CACHE_RESULT adi_cache_Init(void)
{
        return ADI_CACHE_SUCCESS;
}

#endif

int main(int argc, char *argv[])
{
	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */

#if defined(__ADSPCORTEXA55__)
	/* Disable D-Cache, I-Cache */
    register volatile uint32_t sctlr_el3;
    asm volatile ("MRS %0, SCTLR_EL3\n" : "=r" (sctlr_el3) );
    sctlr_el3 &= ~((1 << 2) | (1 << 12));
    asm volatile ("MSR SCTLR_EL3, %0\n" : :"r" (sctlr_el3) );
#endif

    adi_initComponents();

#ifdef Reinitialize_DMC

	/*writing known data to DDR memory */
	for (int i=0;i<BYTE_COUNT/4;i++)
	{
		check_array[i]=i;
	}

    /* Enter no interrupt code*/
    adi_osal_EnterCriticalRegion();

    __asm__("dsb sy");

	if(DMCReInit()!= ADI_DMC_SUCCESS)
	{
		return ADI_DMC_FAILURE;
	}

	__asm__("dsb sy");

    /* Exit no interrupt code*/
    adi_osal_ExitCriticalRegion();

	for (int i=0;i<BYTE_COUNT/4;i++)
	{
		if(check_array[i]!=i)
		{
			printf("\n Re-Initialization test fail");
			break;
		}
	}

	printf("\n All Done");

#endif
	return 0;
}

